import datetime
import urllib

import pandas as pd

FMT_YYYY_MM_DD = "%Y-%m-%d"

def fmt_dt_yyyy_mm_dd(dt: datetime.datetime) -> str:
    return dt.strftime(FMT_YYYY_MM_DD) if dt else None

def url_join(base_url: str, params: dict[str, str]) -> str:
    return base_url + '?' + urllib.parse.urlencode(params)

def parse_dt_yyyy_mm_dd(dtstr: str) -> datetime.datetime:
    return datetime.datetime.strptime(dtstr, FMT_YYYY_MM_DD)

def dump_to_hdf_file(filepath: str, hdf_key: str, df: pd.DataFrame, mode='w'):
    df.to_hdf(filepath, key=hdf_key, mode=mode)

def read_hdf_file(filepath: str, hdf_key: str, mode='r') -> pd.DataFrame:
    return pd.read_hdf(filepath, key=hdf_key, mode=mode)



'''
Data from: https://eodhistoricaldata.com/
'''

from collections import defaultdict
from datetime import datetime
from dataclasses import dataclass

import requests
import pandas as pd

from . import util

@dataclass
class EodHistoricalDataReader:
    api_key: str

    def end_of_day_prices(self, symbol: str,  start: datetime, end: datetime=None, period: str='d'):
        '''
        period: 'd' for daily, 'w' for weekly, 'm' for monthly.
        Returns DataFrame:
            index => date
            Columns:
                open, high, low, close, adjusted_close, volume
        '''
        assert period in ('d', 'w', 'm'), 'Invalid period'

        if end is None:
            end = datetime.now()

        json_resp = self.__json_req(
            api_path='eod',
            symbol=symbol,
            params={
                'period': period,
                'from': util.fmt_dt_yyyy_mm_dd(start),  
                'to': util.fmt_dt_yyyy_mm_dd(end),
            }
        )
        idx = []
        data = defaultdict(list)
        for rec in json_resp:
            idx.append(util.parse_dt_yyyy_mm_dd(rec["date"]))
            for col in ['open', 'high', 'low', 'close', 'adjusted_close', 'volume']:
                data[col].append(rec[col])
        return pd.DataFrame(index=idx, data=data)

    def dividends(self, symbol: str, start: datetime, end: datetime=None) -> pd.DataFrame:
        '''
        Returns DataFrame:
            index => date
            Columns:
            value: number
            currency: str
        '''
        assert start is not None, "Must specify start date"
        if end is None:
            end = datetime.now()
        json_resp = self.__json_req(
            api_path='div',
            symbol=symbol,
            params={
                'from': util.fmt_dt_yyyy_mm_dd(start),  
                'to': util.fmt_dt_yyyy_mm_dd(end),
            }
        )
        idx = []
        data = defaultdict(list)
        for rec in json_resp:
            idx.append(util.parse_dt_yyyy_mm_dd(rec["date"]))
            data['value'].append(rec["value"])
            data['currency'].append(rec["currency"])
        return pd.DataFrame(index=idx, data=data)

    
    def __json_req(self, api_path, symbol, params) -> str:
        return self.__req(
            api_path=api_path,
            symbol=symbol,
            params=params,
            fmt='json'
        ).json()

    def __csv_req(self, api_path, symbol, params) -> str:
        return self.__req(
            api_path=api_path,
            symbol=symbol,
            params=params,
            fmt='csv'
        ).text
    
    def __req(self, api_path, symbol, params, fmt: str) -> str:
        p = dict(params)
        p.update({ 'api_token': self.api_key, 'fmt': fmt })
        url = util.url_join(
            f'https://eodhistoricaldata.com/api/{api_path}/{symbol}',
            p
        )
        return requests.get(url)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['marketdataproviders']

package_data = \
{'': ['*']}

install_requires = \
['pandas>=1.4.1,<2.0.0', 'requests>=2.27.1,<3.0.0']

setup_kwargs = {
    'name': 'marketdataproviders',
    'version': '0.1.0',
    'description': 'APIs around Market Data Providers',
    'long_description': "# Market Data Providers\n\nMiscellaneous APIs around Market Data providers.\n\n## APIs\n\nCurrently only basic access to: <https://eodhistoricaldata.com/> is implemented.\n\n\n## Build\n\nThis is built using [Poetry](https://python-poetry.org/).\n\n```bash\n# after checkout, within top-level checkout directory\npoetry install\n\n# To update dependencies, or after `poetry add ...`\npoetry update\n\n# to make sure we are using our virtual env's tooling (created\n# automatically by poetry, much like conda env)\n# We can, for example add ipython as a dev dependency, then shell into our\n# env, only then will we get this install. Use `which ipython` to verify.\npoetry shell\n\n# To build distributable packages\npoetry build\n```\n\n## Publishing\n\n### To PyPi\n\nCreate PyPI account with a new token.\n\nAdd this to poetry's config:\n\n```\npoetry config pypi-token.pypi <my-token>\n```\n\nSimply publish:\n\n```\npoetry publish home-pypi\n```\n\n\n### To Local Repository\n\nOnce off only: Prepare Local Repository\n\nSee: \n\n* https://pypi.org/project/pypiserver/: Install this.\n* https://python-poetry.org/docs/master/repositories/\n\n\n```bash\n# Create location\nLOCAL_REPO_FS=$HOME/.local/home-pypi\nmkdir -p $LOCAL_REPO_FS\ncd $LOCAL_REPO_FS\n\n# Run pypi-server in local repo fs, unauthorised mode.\npypi-server -i 127.0.0.1 -v -P . -a . -p 9292 .\n```\nAdd repository name to `poetry` config:\n\n```bash\npoetry config repositories.home-pypi http://localhost:9292/\n```\nUpdate `pip` configuration to use this extra index.\n\n```\n# ~/.pip/pip.conf -> https://pypi.org/project/pypiserver/#configuring-pip\n# Add the following config\n\n[global]\nextra-index-url = http://localhost:9292/simple/\n```\n\nTo publish to local repo:\n\n```\n# Publish distributable packages to local repo set-up with name above.\n# Just press Enter when username asked, we don't have authentication on.\npoetry publish --repository home-pypi\n```",
    'author': 'Kamal Advani',
    'author_email': 'kamal.advani@namingcrisis.net',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
